@@ -237,6 +237,7 @@ class Connection:
                     # Manually clear the token and retry once. The token is supposed to be automatically
                     # refreshed, but if the local clock is off that can mess things up.
                     self._oauth2_token = None
+                    self._rewind_request_data_if_needed(data)
                     response = self.session.put(url, data=data, **session_put_kwargs)
 
                 if allowable_response_status_codes:
@@ -247,6 +248,8 @@ class Connection:
                 if retries <= 0:
                     raise e
 
+                self._rewind_request_data_if_needed(data)
+
                 retries -= 1
                 self.interruptable_sleep(retry_delay)
 
@@ -347,7 +350,8 @@ class Connection:
                 retries -= 1
                 self.interruptable_sleep(retry_delay)
 
-    def do_post_request(self, url, allowable_response_status_codes=(200,), retries=None, retry_delay=None, **kwargs):
+    def do_post_request(self, url, allowable_response_status_codes=(200,), retries=None, retry_delay=None,
+                        data=None, **kwargs):
         assert url is not None
         session_post_kwargs = self._get_requests_kwargs()
         session_post_kwargs.update(kwargs)
@@ -361,12 +365,13 @@ class Connection:
         while True:
             try:
                 try:
-                    response = self.session.post(url, **session_post_kwargs)
+                    response = self.session.post(url, data=data, **session_post_kwargs)
                 except TokenExpiredError:
                     # Manually clear the token and retry once. The token is supposed to be automatically
                     # refreshed, but if the local clock is off that can mess things up.
                     self._oauth2_token = None
-                    response = self.session.post(url, **session_post_kwargs)
+                    self._rewind_request_data_if_needed(data)
+                    response = self.session.post(url, data=data, **session_post_kwargs)
 
                 if allowable_response_status_codes:
                     utils.validate_response_is_ok(response, allowable_response_status_codes)
@@ -376,9 +381,15 @@ class Connection:
                 if retries <= 0:
                     raise e
 
+                self._rewind_request_data_if_needed(data)
+
                 retries -= 1
                 self.interruptable_sleep(retry_delay)
 
+    def _rewind_request_data_if_needed(self, data):
+        if isinstance(data, io.BytesIO):
+            data.seek(0)
+
     def do_delete_request(self, url, allowable_response_status_codes=(200,), retries=None, retry_delay=None, **kwargs):
         assert url is not None
         session_delete_kwargs = self._get_requests_kwargs()
