@@ -4,13 +4,14 @@ import copy
 import logging
 
 import io
+import pprint
+import textwrap
 import time
 
 from . import entity_json
 from .exceptions import ConfigUploadFailed
 from .entitybase import EntityBase
 from .pump import Pump
-from . import utils
 
 logger = logging.getLogger(__name__)
 
@@ -192,7 +193,7 @@ class Pipe(EntityBase):
 
         self.update_raw_jsondata(response.content.decode("utf-8"))
 
-    def wait_for_pipe_to_be_deployed(self, timeout=60, **kwargs):
+    def wait_for_pipe_to_be_deployed(self, timeout=300, **kwargs):
         starttime = time.monotonic()
         while True:
             self.update_raw_jsondata_from_url(self._connection.get_pipe_url(self.id), **kwargs)
@@ -200,5 +201,7 @@ class Pipe(EntityBase):
                 return
             elapsetime = time.monotonic() - starttime
             if elapsetime > timeout:
-                raise AssertionError("Timed out while waiting for the config of the pipe '{self.id}' to be deployed.")
+                raise AssertionError(
+                    f"Timed out while waiting for the config of the pipe '{self.id}' to be deployed. "
+                    f"self._raw_jsondata: {textwrap.indent(pprint.pformat(self._raw_jsondata), prefix='    ')}")
             time.sleep(1)
